#!/usr/bin/python
# -*- coding: utf-8 -*-

# commonFunctions.py
#
# This file is part of BotMU
#
# Author: Raphaël Perraudeau <r.perraudeau@gmail.com>
# Copyright: 2011 Raphaël Perraudeau <r.perraudeau@gmail.com>
# Licence: GNU/GPL
#
# Home page: <http://botmu.sourceforge.net/>


import datetime
import subprocess
import platform
import os
import sys
import inspect
from urllib2 import unquote

from htmlentitydefs import name2codepoint as n2cp
import re

from options import *

if platform.system() == 'Windows':
	try:
		import _winreg
	except:
		pass

# Fonction d'ajout de la ligne de la session dans le fichier log
#========================================================================
def logErrorInitSession():
	configDir = getConfDir()
	logInfos = "\n#####\n"
	logInfos += "# %s\n" % datetime.datetime.now()
	logInfos += "#####\n"
	logFile = os.path.join(configDir, "BotMU.exe.log")
	try:
		lf = open(logFile, "a+")
		lf.write(logInfos)
		lf.close()
	except:
		pass

# Fonction de log
#========================================================================
def logError(errorPosition, errorType, errorValue):

	configDir = getConfDir()
	logInfos = "[%s] Error [%s]: %s - %s\n" % (datetime.datetime.now(), errorPosition, errorType, errorValue)
	logFile = os.path.join(configDir, "BotMU.exe.log")
	try:
		lf = open(logFile, "a+")
		lf.write(logInfos)
		lf.close()
	except:
		pass

# Emplacement du dossier de configuration
#========================================================================
def getConfDir():
	# Linux:
	if platform.system() == 'Linux':
		configDir = os.path.join(os.path.expanduser("~"), ".BotMU")
		if(not os.path.exists(configDir) or not os.path.isdir(configDir)):
			try:
				os.makedirs(configDir)
			except:
				logError("commonFunctions - getConfDir - 1", sys.exc_info()[0], sys.exc_info()[1])
				return ""
		return configDir

	# Windows: Fonctionne avec XP, à tester avec Vista/Seven
	if platform.system() == 'Windows':
		configDir = os.path.join(os.environ['APPDATA'], 'BotMU')
		if(not os.path.exists(configDir) or not os.path.isdir(configDir)):
			try:
				os.makedirs(configDir)
			except:
				logError("commonFunctions - getConfDir - 2", sys.exc_info()[0], sys.exc_info()[1])
				return ""
		return configDir
			
	# FIXME Mac
	else:
		return "./"

		
# Emplacement du dossier d'execution
#========================================================================
def getExecDir():
	return os.path.dirname(sys.argv[0])
		
		
# Emplacement du dossier perso
#========================================================================
def getUserPersonalPath():
	# Linux:
	if platform.system() == 'Linux':
		return os.path.expanduser("~")

	# Windows: Fonctionne avec XP, à tester avec Vista/Seven
	if platform.system() == 'Windows':
		key = _winreg.OpenKey( _winreg.HKEY_CURRENT_USER, 'Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders',0, _winreg.KEY_READ)
		(valeur,typevaleur) = _winreg.QueryValueEx(key,'Personal')
		_winreg.CloseKey(key)
		return valeur
	
	else:
		return os.path.expanduser("~")
	

# Emplacement du dossier perso
#========================================================================
def openDirInNavigator(filepath):
	if platform.system() == 'Linux':
		subprocess.call(('xdg-open', filepath))
	elif platform.system() == 'Windows':
		os.startfile(filepath)
	else:
		subprocess.call(('open', filepath))
	

# fonction isint
#========================================================================
def isInt(value):
	try:
		if int(value) == value:
			return True
		else:
			return False
	except:
		return False

# effacer le shell
#========================================================================
def clearShell():
	# Linux:
	if platform.system() == 'Linux':
		os.system("clear")

	# Windows: Fonctionne avec XP, à tester avec Vista/Seven
	if platform.system() == 'Windows':
		os.system("cls")
			
	# FIXME Mac
	else:
		os.system("clear")

# Sous fonction de htmlentities_decode
#========================================================================
def substitute_entity(match):
	ent = match.group(3)
	if match.group(1) == "#":
		if match.group(2) == '':
			return unichr(int(ent))
		elif match.group(2) == 'x':
			return unichr(int('0x'+ent, 16))
	else:
		cp = n2cp.get(ent)

		if cp:
			return unichr(cp)
		else:
			return match.group()

# Fonction htmlentities_decode
#========================================================================
def htmlentities_decode(string):
	entity_re = re.compile(r'&(#?)(x?)(\w+);')
	return entity_re.subn(substitute_entity, string)[0]

# Fonction de retour de nom de la fontion en cours
#========================================================================
def whoami():
	return inspect.stack()[1][3]

# fonction url decode
#========================================================================
def urldecode(string):
	return unquote(string)

# Conversion de seconde en temps humain
#========================================================================
def secToTime(sec):
	#semaine: out
	if(sec >= 3600*24*7):
		timeLeft = "Plus d'une semaine"
	#jour
	elif(sec >= 3600*24):
		timeDay = int(sec/(3600*24))
		timeSec = sec%(3600*24)
		timeHour = int(timeSec/3600)
		timeSec = sec%3600
		timeMin = int(timeSec/60)
		timeSec = timeSec%60
		timeLeft = "%sj %sh %sm %ss" % (timeDay, timeHour, timeMin, timeSec)
	#heure
	elif(sec >= 3600):
		timeHour = int(sec/3600)
		timeSec = sec%3600
		timeMin = int(timeSec/60)
		timeSec = timeSec%60
		timeLeft = "%sh %sm %ss" % (timeHour, timeMin, timeSec)

	#minutes
	elif(sec >= 60):
		timeSec = sec%60
		timeMin = int(sec/60)
		timeLeft = "%sm %ss" % (timeMin, timeSec)
	#secondes
	else:
		timeLeft = "%ss" % int(sec)
	return timeLeft
	
def cutString(value, nb=50):
	if len(value)>nb:
		value = "%s [...] %s" % (value[0:22], value[-22:len(value)])
	return value
