#!/usr/bin/python
# -*- coding: utf-8 -*-

# httpReq.py
#
# This file is part of BotMU
#
# Author: Raphaël Perraudeau <r.perraudeau@gmail.com>
# Copyright: 2011 Raphaël Perraudeau <r.perraudeau@gmail.com>
# Licence: GNU/GPL
#
# Home page: <http://botmu.sourceforge.net/>


import urllib2
import cookielib
import socket
import os
import sys
import time
import datetime
import tempfile

from commonFunctions import *


#############################################################################
class httpReq:

	#========================================================================
	def __init__(self, confDir):
		self.proxy = ''
		self.referer 			= ''
		self.cookie 			= ''
		self.useragent 			= ''
		self.method 			= ''
		self.data 				= ''
		self.verbose 			= 0
		self.continuedl 		= 0
		self.limit				= 0
		self.execTime 			= 0.0
		self.downloadedOctets	= 0
		self.lasttime 			= 0
		self.downloadSpeed 		= 0
		self.pause				= False
		self.stopthisfile 		= False
		self.configDir			= confDir


	# getPage : contacter un url, et récupérer le html 
	#========================================================================
	def getPage(self, url, dest=False, report_hook=False, chunk_size=2048):
		
		socket.setdefaulttimeout(10)
		
		if self.verbose==1:
			print "###############"
			print "# getPage: %s" % url
		try:
			domaineName = url.split("://")[1].split("/")[0]
			domaineName = "%s.%s" % (domaineName.split(".")[-2], domaineName.split(".")[-1])
		except:
			domaineName = "unknow"
		COOKIEDIR  = os.path.join(tempfile.gettempdir(), 'BotMU/cookies/')
		COOKIENAME = '%s.lwp' % domaineName
		COOKIEFILE = '%s%s' % (COOKIEDIR, COOKIENAME)
		cj = cookielib.LWPCookieJar()
		if os.path.isfile(COOKIEFILE):
			if os.path.getsize(COOKIEFILE) > 1:
				cj.load(COOKIEFILE)

		try:
			url = str(url)
		except:
			return -2
				
		"""Set data to send:"""
		if self.data == '':
			req = urllib2.Request(url)
		else:
			req = urllib2.Request(url, self.data)

		"""Set a proxy:"""
		if self.proxy == '':
			req_proxy = urllib2.ProxyHandler({})
			if self.verbose == 1:
				print "# No Proxy Used"
		else:
			proto = url.split("://")[0]
			req_proxy = urllib2.ProxyHandler({proto : self.proxy})
			if self.verbose == 1:
				print "# Proxy: ", self.proxy

		"""Set cookies:"""
		if self.cookie =='':
			req_cookie = urllib2.HTTPCookieProcessor(cj)
			opener = urllib2.build_opener(req_proxy, req_cookie)
			if self.verbose == 1:
				print "# Cookie auto"
		elif self.cookie == False:
			opener = urllib2.build_opener(req_proxy)
			if self.verbose == 1:
				print "# Forced No Cookie"
		else:
			opener = urllib2.build_opener(req_proxy)
			req.add_header('Cookie', self.cookie)
			if self.verbose == 1:
				print "# Forced Cookie"

		"""Set useragent:"""
		if self.useragent == '':
			if self.verbose == 1:
				print "# No User Agent Sended"
		else:
			req.add_header('User-Agent', self.useragent)
			if self.verbose == 1:
				print '# User-Agent: ',self.useragent

		"""Set a referer:"""
		if self.referer == '':
			if self.verbose == 1:
				print "# No Referer Sended"
		else:
			req.add_header('Referer', self.referer)
			if self.verbose == 1:
				print '# Referer: ',self.referer
				
		"""Set a range:"""
		if self.continuedl == 0:
			if self.verbose == 1:
				print "# No Range Sended"
		else:
			if dest:
				try:
					fileSize = os.path.getsize(dest)
					bytesRange = 'bytes=%s-' % (fileSize)
					req.add_header('Range', bytesRange)
					if self.verbose == 1:
						print '# Range: %s' % (bytesRange)
				except:
					logError("httpReq - getPage - 1", sys.exc_info()[0], sys.exc_info()[1])
					if self.verbose == 1:
						print '# Range: Local file deleted, Impossible to continue download...'
		try:
			page = opener.open(req)
			# Implémentation: enregistrement de fichier
			# Implémentation: reporthook pour urllib2
			# Implémentation: reprise d'un téléchargement
			# Implémentation: limite de debits
			if dest:
				bytes_so_far = 0
				if self.continuedl == 0:
					destDir = os.path.dirname(dest)
					if destDir!="" and not os.path.isdir(destDir):
						os.makedirs(destDir)
					try:
						saveFile = open(dest, "w+")
						saveFile.close()
					except:
						pass
					modeOpen = "wb"
					bytes_so_far = 0
				else:
					modeOpen = "ab"
					try:
						bytes_so_far = os.path.getsize(dest)
					except:
						logError("httpReq - getPage - 2", sys.exc_info()[0], sys.exc_info()[1])
						bytes_so_far = 0
				
				
				total_size = page.info().getheader('Content-Length').strip()
				total_size = int(total_size) + bytes_so_far
				
				
				with open(dest, modeOpen) as saveFile:
					chunk = page.read(chunk_size)
					while chunk and not self.stopthisfile:
						while self.pause:
							time.sleep(1)
						saveFile.write(chunk)
						saveFile.flush() # Pas obligatoire, cela forcera l’écriture réelle du chunk dans le fichier.
						bytes_so_far += len(chunk)
						if report_hook:
							self.reportHook(report_hook, int(bytes_so_far/chunk_size), chunk_size, total_size)
						chunk = page.read(chunk_size)
					if self.stopthisfile:
						self.stopthisfile = False
			else:
				if report_hook:
					total_size = page.info().getheader('Content-Length').strip()
					total_size = int(total_size)
					bytes_so_far = 0
					while 1:
						chunk = page.read(chunk_size)
						bytes_so_far += len(chunk)
						if not chunk:
							break
						self.reportHook(report_hook, int(bytes_so_far/chunk_size), chunk_size, total_size)
		
		except IOError, e:
			if self.verbose==1:
				print ' '
				print '# Echec lors de la lecture HTTP. '
				print '# -----Details-----'
				print '# URL ', url
				print '# Erreur :', e
			if hasattr(e, 'code'):
				if self.verbose==1:
					print 'code', e.code
				return e.code
			elif hasattr(e, 'reason'):
				if self.verbose==1:
					print 'raison', e.reason
				return e.reason
			else:
				logError("httpReq - getPage - 3", sys.exc_info()[0], "%s - %s" % (sys.exc_info()[1], url))
				return e

		except:
			if self.verbose==1:
				print ' '
				print '# Echec lors de la lecture HTTP. '
				print '# -----Details-----'
				print '# URL ', url
				print '# Erreur :', sys.exc_info()[0]
				print '------------------'
			if "Abort" in "%s" % sys.exc_info()[0]:
				return -1
			else:
				logError("httpReq - getPage - 4", sys.exc_info()[0], sys.exc_info()[1])
				return -3
		
		if cj is not None:
			if not os.path.isfile(COOKIEFILE):
				if not os.path.isdir(COOKIEDIR):
					os.makedirs(COOKIEDIR)
				try:
					saveFile = open(COOKIEFILE, "w+")
					saveFile.close()
				except:
					pass
			if os.path.isfile(COOKIEFILE):
				cj.save(COOKIEFILE)
		return page

		
		
	# Report Hook
	#========================================================================
	def reportHook(self, reporter, telechbloc, taillebloc, totalblocs):
		reporter(telechbloc, taillebloc, totalblocs)
		
		
		
		#calcul de la vitesse de téléchargement
		if(self.lasttime!=0):
			tmpDLSpeed = int(self.downloadedOctets/1024)
			if(self.execTime>=1):
				#on trouve la vitesse de téléchargement
				downloadSpeedOct = int(self.downloadedOctets)
				self.downloadSpeed = int(self.downloadedOctets/1024)
				#et on repart a 0
				self.execTime=0.0
				self.downloadedOctets = 0
				tmpDLSpeed = 0
			elif(tmpDLSpeed>(self.limit-10) and self.limit!=0):
				time.sleep(float(1-self.execTime))
				tmpDLSpeed = 0
				self.execTime=0.0
				self.downloadedOctets = 0
			else:
				timeNow = time.time()
				timeExe = timeNow-self.lasttime
				self.downloadedOctets = self.downloadedOctets+taillebloc
				self.execTime=self.execTime+timeExe
		self.lasttime = time.time()
		

	# Fonctions de définition de paramètres de la requète
	#========================================================================

	"""Proxy"""
	def defineProxy(self, proxy):
		self.proxy = proxy
		return 1
	"""Referer"""
	def defineReferer(self, referer):
		self.referer = referer
		return 1
	"""Cookie"""
	def defineCookie(self, cookie):
		self.cookie = cookie
		return 1
	"""UserAgent"""
	def defineUseragent(self, useragent):
		self.useragent = useragent
		return 1
	"""Verbose, Debug only =)"""
	def defineVerbose(self, value):
		self.verbose = value
	"""Forcer la method de requete (post, get, ou autre (.htaccess))"""
	def defineForceMethod(self, value):
		self.method = value
	"""Data to post"""
	def defineData(self, value):
		self.data = value
	"""Range of download"""
	def defineContinue(self, value):
		self.continuedl = value
	"""Rate Limiting"""
	def defineLimit(self, value):
		self.limit = value
